package dynaaminen.gen;

/**
 * Esimerkki dynaamisesta taulukosta
 * @author Vesa Lappalainen @version 1.0, 02.03.2002
 * @param <TYPE> minkä tyyppisiä alkioita tallennetaan
 * @example
 * <pre name="test">
 *  #THROWS Taulukko.TaulukkoTaysiException
 *  Taulukko luvut = new Taulukko();
 *  luvut.lisaa(0.0); luvut.lisaa(2.0); luvut.lisaa(99.0);
 *  luvut.toString() === "0.0,2.0,99.0";
 *  luvut.set(1,4); luvut.toString() === "0.0,4.0,99.0";
 *  double luku = luvut.get(2);
 *  luku ~~~ 99;
 *  luvut.get(2) ~~~ 99;
 *  luvut.set(21, 4.0); #THROWS IndexOutOfBoundsException
 *  luvut.lisaa(0.0); luvut.lisaa(0.0); //taulukko täyteen
 *  luvut.lisaa(0.0);  #THROWS Taulukko.TaulukkoTaysiException
 * </pre>
 */
public class Taulukko<TYPE> {

    /** Luokka täyden taulukon poikkeusta varten  */
    public static class TaulukkoTaysiException extends Exception {
        private static final long serialVersionUID = 1L;


        /**
         * @param viesti joka on poikkeuksen mukana
         */
        public TaulukkoTaysiException(String viesti) {
            super(viesti);
        }
    }

    private int lkm = 0;
    private TYPE[] alkiot;
    
    /**
     * Alustetaan 5 kokoinen taulukko
     */
    public Taulukko() {
        //alkiot = new int[5];
        this(5);
    }


    /**
     * Alutetaan valitun kokoinen taulukko
     * @param koko taulukon koko
     */
    @SuppressWarnings("unchecked")
    public Taulukko(int koko) {
        alkiot = (TYPE[])new Object[koko];
    }

    
    /**
     * Lisätään taulukkoon yksi alkio
     * @param luku lisättävä alkio
     * @throws TaulukkoTaysiException jos tila loppuu
     */ 
    public void lisaa(TYPE luku) throws TaulukkoTaysiException {
        if (alkiot.length <= lkm) throw new TaulukkoTaysiException("Tila loppui " + luku);
        alkiot[lkm] = luku;
        lkm++;
    }

    
    @Override
    public String toString() {
        StringBuilder s = new StringBuilder("");
        String erotin = "";
        for (int i = 0; i < lkm; i++) {
            s.append(erotin);
            s.append(alkiot[i]);
            erotin = ",";
        }
        return s.toString();
    }

    
    /**
     * Asetetaan taulukon i's alkio
     * @param i mihin paikkaan astetaan
     * @param alkio mikä arvo laitetaan
     * @throws IndexOutOfBoundsException jos väärä indeksi
     */
    public void set(int i, TYPE alkio) throws IndexOutOfBoundsException {
        if ((i < 0) || (lkm <= i)) throw new IndexOutOfBoundsException("i = " + i);
        alkiot[i] = alkio;
    }


    /**
     * @param i mistä kohti
     * @return paikassa i oleva arvo
     * @throws IndexOutOfBoundsException jos indeksi väärin
     */
    public TYPE get(int i) throws IndexOutOfBoundsException  {
        if ((i < 0) || (lkm <= i)) throw new IndexOutOfBoundsException("Laiton indeksi i = " + i);
        return alkiot[i];
    }
    
    
    /**
     * Kokeillaan taulukkoa
     * @param args ei käytössä
     */
    public static void main(String[] args) {
        Taulukko<String> sanat = new Taulukko<String>(5);
        try {
            sanat.lisaa("kissa");
            sanat.lisaa("koira");
            sanat.lisaa("sammakko");
            // luvut.lisaa(199);
            // luvut.lisaa(299);
            // luvut.lisaa(399);
            String a = sanat.get(1);
            a = sanat.get(3);
            System.out.println(a);  // 2
        } catch (TaulukkoTaysiException e) {
            System.err.println(e.getMessage());
        } catch (IndexOutOfBoundsException e) {
            System.err.println(e.getMessage());
        }
        System.out.println(sanat);
        
        Taulukko<Integer> luvut = new Taulukko<>(); 
        try {
            luvut.lisaa(Integer.valueOf(0));  // autoboxin
            luvut.lisaa(2);
            luvut.lisaa(99);
        } catch (TaulukkoTaysiException e) {
            System.err.println(e.getMessage());
        }
        
        int luku = luvut.get(2);
        System.out.println(luku);
        System.out.println(luvut);
        
        Taulukko<Double> luvutd = new Taulukko<>(); 
        try {
            luvutd.lisaa(Double.valueOf(0.0));
            luvutd.lisaa(2.0);
            luvutd.lisaa(99.0);
        } catch (TaulukkoTaysiException e) {
            System.err.println(e.getMessage());
        }
        
        System.out.println(luvutd);
        
        
    }    
}
