package dynaaminen;

import fi.jyu.mit.ohj2.Tiedosto;

import java.io.OutputStream;
import java.io.PrintStream;
import java.util.Iterator;
import java.util.NoSuchElementException;

/**
 * Esimerkki dynaamisesta taulukosta ja iteraattorista
 * @author Vesa Lappalainen
 * @version 1.0, 02.03.2002
 * @version 1.1, 26.02.2012
 */

public class TaulukkoIter implements Iterable<Integer> {
    /** Luokka täyden taulukon poikkeusta varten  */
    public class TaulukkoTaysiException extends Exception {
        private static final long serialVersionUID = 1L;
        TaulukkoTaysiException(String viesti) {
            super(viesti);
        }
    }

    private Integer alkiot[];
    private int lkm;


    /**
     * @return taulukon alkioiden lukumäärä
     * @example
     * <pre name="test">
     *   #THROWS TaulukkoIter.TaulukkoTaysiException
     *   TaulukkoIter t = new TaulukkoIter();
     *   t.size() === 0;
     *   t.lisaa(2);   t.size() === 1;
     *   t.lisaa(99);  t.size() === 2;
     * </pre>
     */
    public int size() {
        return lkm;
    }
    
    
    /**
     * Alustetaan 5 kokoinen taulukko 
     */
    public TaulukkoIter() {
        alkiot = new Integer[5];
        lkm = 0;
    }


    /**
     * Alutetaan valitun kokoinen taulukko
     * @param koko taulukon koko
     */
    public TaulukkoIter(int koko) {
        alkiot = new Integer[koko];
        lkm = 0;
    }


    /**
     * Lisätään taulukkon uusi alkio
     * @param alkio lisättävän alkion viite
     * @throws TaulukkoTaysiException jos taulukko jo täysi 
     * @example
     * <pre name="test">
     *   #THROWS TaulukkoIter.TaulukkoTaysiException
     *   TaulukkoIter t = new TaulukkoIter(2);
     *   t.size() === 0;
     *   t.lisaa(2);   t.size() === 1;
     *   t.lisaa(99);  t.size() === 2;
     *   t.lisaa(5); #THROWS TaulukkoIter.TaulukkoTaysiException
     * </pre>
     */
    public void lisaa(Integer alkio) throws TaulukkoTaysiException {
        if (lkm >= alkiot.length) throw new TaulukkoTaysiException("Tila loppu");
        alkiot[lkm] = alkio;
        lkm++;
    }


    /**
     * @return taulukko merkkijonona välilyönneinen eroteltuna
     * @example
     * <pre name="test">
     *   #THROWS TaulukkoIter.TaulukkoTaysiException
     *   TaulukkoIter t = new TaulukkoIter();
     *   t.lisaa(2);  
     *   t.lisaa(99); 
     *   t.lisaa(5); 
     *   t.toString() === " 2 99 5";
     * </pre>
     */
    @Override
    public String toString() {
        StringBuilder s = new StringBuilder("");
        for (int i = 0; i < lkm; i++) 
            s.append(" " + alkiot[i]);
        return s.toString();
    }


    /**
     * Muutetaan paikan i arvo
     * @param i mihin paikkaa asetetaan
     * @param alkio uuden alkion viite
     * @throws IndexOutOfBoundsException jos indeksi väärin
     * @example
     * <pre name="test">
     *   #THROWS TaulukkoIter.TaulukkoTaysiException
     *   TaulukkoIter t = new TaulukkoIter();
     *   t.lisaa(2);  t.lisaa(99);   t.lisaa(5);
     *   t.set(1,88); t.set(2,6);
     *   t.set(3,77); #THROWS IndexOutOfBoundsException 
     *   t.toString() === " 2 88 6";
     * </pre>
     */
    public void set(int i, Integer alkio) throws IndexOutOfBoundsException {
        if ( ( i < 0 ) || ( lkm <= i ) ) throw new IndexOutOfBoundsException("i = " + i);
        alkiot[i] = alkio;
    }


    /**
     * Paikassa i olevan alkion viite
     * @param i mistä paikasta
     * @return paikassa i oleva viite
     * @throws IndexOutOfBoundsException jos indeksi väärin
     * @example
     * <pre name="test">
     *   #THROWS TaulukkoIter.TaulukkoTaysiException
     *   TaulukkoIter t = new TaulukkoIter();
     *   t.lisaa(2);   t.get(0) === new Integer(2);  
     *   t.lisaa(99);  t.get(1) === new Integer(99); 
     *   t.lisaa(5);   t.get(2) === new Integer(5);
     *   t.get(3); #THROWS IndexOutOfBoundsException 
     * </pre>
     */
    public Integer get(int i) throws IndexOutOfBoundsException {
        if ((i < 0) || (lkm <= i)) throw new IndexOutOfBoundsException("i = " + i);
        return alkiot[i];
    }


    /**
     * Tulostetaan taulukko tietovirtaan
     * @param os mihin virtaan tulostetaan
     * @param luvut tulostettavat luvut
     * @example
     * <pre name="test">
     *  #THROWS TaulukkoIter.TaulukkoTaysiException
     *  #import java.io.*;
     *   ByteArrayOutputStream bs = new ByteArrayOutputStream();
     *   TaulukkoIter t = new TaulukkoIter();
     *   t.lisaa(2); t.lisaa(99); t.lisaa(5);
     *   tulosta(bs,t);
     *   bs.toString() =R= "(?s)2 99 5 .*";   
     * </pre>
     */
    public static void tulosta(OutputStream os, TaulukkoIter luvut) {
        @SuppressWarnings("resource")
        PrintStream out = Tiedosto.getPrintStream(os);
        /*
        for (Iterator<Integer> i = luvut.iterator(); i.hasNext();) {
            int luku = i.next().intValue();
            out.print(luku + " ");
        }
        */
        // for (int luku : luvut) out.print(luku + " ");
        out.println(luvut);
    }


    /**
     * Iteraattori lukujen läpikäymiseksi
     */
    public class Iter implements Iterator<Integer> {
        int kohdalla;
        
        /**
         * Alustetaan alkuun 
         */
        public Iter() {
            kohdalla = 0;
        }
        
        
        @Override
        public boolean hasNext() {
            // if ( kohdalla >= size()) return false;
            // return true;
            return kohdalla < size();
        }

        @Override
        public Integer next() {
            if (!hasNext()) throw new NoSuchElementException("Ei oo enää");
            Integer alkio = get(kohdalla);
            kohdalla++;
            return alkio;
            // return get(kohdalla++);
            // return get(++kohdalla);
        }
        
    }

    
    
    
    @Override
    public Iterator<Integer> iterator() {
        return new Iter();
    }
    
    
    
    /**
     * Kokeillaan iteraattoria
     * @param args ei käytössä
     */
    public static void main(String[] args) {
        /*
         * Integer i1 = new Integer(5); Integer i2 = new Integer(6);
         * System.out.println(i1); i2 = new Integer(4); i2 = 4; int i3 =
         * i2.intValue(); i2 = i1 * i2; i2 = new Integer(i1.intValue() +
         * i2.intValue()); System.out.println(i2);
         * 
         * if ( true ) return;
         */
        TaulukkoIter luvut = new TaulukkoIter(100);
        try {
            luvut.lisaa(0); // luvut.lisaa(new Integer(0));
            luvut.lisaa(2);
            luvut.lisaa(99);
        } catch (TaulukkoTaysiException e) {
            System.out.println("Virhe: " + e.getMessage());
        }
        
        System.out.println(luvut);
        luvut.set(1, 4); // luvut.set(1, new Integer(4));
        System.out.println(luvut);
        int luku2 = luvut.get(2);
        System.out.println("Paikassa 2 on " + luku2);

        tulosta(System.out, luvut);

        try {
            luvut.set(21, 4);
        } catch (IndexOutOfBoundsException e) {
            System.out.println("Virhe: " + e.getMessage());
        }
        
        // Pidempi versio iteraattorilla läpikäynistä
        for (var iter = luvut.iterator(); iter.hasNext();) {
            int luku = iter.next(); 
            System.out.print(luku + " ");
        }
        System.out.println();
        
        // Saman lyhennetty versio
        for (int luku : luvut)
            System.out.print(luku + " ");
        System.out.println();

    }


}
