package kerho;

import java.io.File;
import java.util.Collection;
import java.util.List;

/**
 * CRC sisällöt tähän
 * @author vesal
 * @version 23.2.2021
 */
public class Kerho {

    private Jasenet jasenet = new Jasenet(); 
    private Harrastukset harrastukset = new Harrastukset();

    private String hakemisto = "kelmit";
    
    /**
     * Lisätään uusi jäsen
     * @param jasen lisättävä jäsen
     * @throws SailoException  jos lisääminen ei onnistu
     */ 
    public void lisaa(Jasen jasen) throws SailoException {
        jasenet.lisaa(jasen);
    }
    
    
    /** 
     * Korvaa jäsenen tietorakenteessa.  Ottaa jäsenen omistukseensa. 
     * Etsitään samalla tunnusnumerolla oleva jäsen.  Jos ei löydy, 
     * niin lisätään uutena jäsenenä. 
     * @param jasen lisätäävän jäsenen viite.  Huom tietorakenne muuttuu omistajaksi 
     * @throws SailoException jos tietorakenne on jo täynnä 
     */ 
    public void korvaaTaiLisaa(Jasen jasen) throws SailoException { 
        jasenet.korvaaTaiLisaa(jasen); 
    } 

    
    /** 
     * Korvaa harrastuksen tietorakenteessa.  Ottaa harrastuksen omistukseensa. 
     * Etsitään samalla tunnusnumerolla oleva harrastus.  Jos ei löydy, 
     * niin lisätään uutena harrastuksena. 
     * @param harrastus lisärtävän harrastuksen viite.  Huom tietorakenne muuttuu omistajaksi 
     * @throws SailoException jos tietorakenne on jo täynnä 
     */ 
    public void korvaaTaiLisaa(Harrastus harrastus) throws SailoException { 
        harrastukset.korvaaTaiLisaa(harrastus); 
    } 


    /**
     * Listään uusi harrastus kerhoon
     * @param har lisättävä harrastus 
     */
    public void lisaa(Harrastus har) {
        harrastukset.lisaa(har);    
    }
  
    
    /**
     * Haetaan kaikki jäsen harrastukset
     * @param jasen jäsen jolle harrastuksia haetaan
     * @return tietorakenne jossa viiteet löydetteyihin harrastuksiin
     * @example
     * <pre name="test">
     * #import java.util.*;
     * 
     *  Kerho kerho = new Kerho();
     *  Jasen aku1 = new Jasen(), aku2 = new Jasen(), aku3 = new Jasen();
     *  aku1.rekisteroi(); aku2.rekisteroi(); aku3.rekisteroi();
     *  int id1 = aku1.getTunnusNro();
     *  int id2 = aku2.getTunnusNro();
     *  Harrastus pitsi11 = new Harrastus(id1); kerho.lisaa(pitsi11);
     *  Harrastus pitsi12 = new Harrastus(id1); kerho.lisaa(pitsi12);
     *  Harrastus pitsi21 = new Harrastus(id2); kerho.lisaa(pitsi21);
     *  Harrastus pitsi22 = new Harrastus(id2); kerho.lisaa(pitsi22);
     *  Harrastus pitsi23 = new Harrastus(id2); kerho.lisaa(pitsi23);
     *  
     *  List<Harrastus> loytyneet;
     *  loytyneet = kerho.annaHarrastukset(aku3);
     *  loytyneet.size() === 0; 
     *  loytyneet = kerho.annaHarrastukset(aku1);
     *  loytyneet.size() === 2; 
     *  loytyneet.get(0) == pitsi11 === true;
     *  loytyneet.get(1) == pitsi12 === true;
     *  loytyneet = kerho.annaHarrastukset(aku2);
     *  loytyneet.size() === 3; 
     *  loytyneet.get(0) == pitsi21 === true;
     * </pre> 
     */
    public List<Harrastus> annaHarrastukset(Jasen jasen) {
        return harrastukset.annaHarrastukset(jasen.getTunnusNro());
    }

    
    /**
     * @return jäsenten lkm
     */
    public int getJasenia() {
        return jasenet.getLkm();
    }
    
    
    /**
     * Antaa kerhon i:n jäsenen
     * @param i monesko jäsen (alkaa 0:sta)
     * @return jäsen paikasta i
     */
    public Jasen annaJasen(int i) {
        return jasenet.anna(i);
    }
    
    
    /**
     * Palauttaa listan jäsenistä jotka vastaavat hakuehtoon
     * @param ehto hakuehto
     * @param k kentän indeksi jonka mukaan etsitään
     * @return löytyneet jäsenet
     */
    public Collection<Jasen> etsi(String ehto, int k) {
        return jasenet.etsi(ehto, k);
    }

    
    /**
     * Lukee kerhon tiedot tiedostosta
     * @param nimi jota käyteään lukemisessa
     * @throws SailoException jos lukeminen epäonnistuu
     */
    public void lueTiedostosta(String nimi) throws SailoException {
        File dir = new File(nimi);
        dir.mkdir();
        jasenet = new Jasenet(); // jos luetaan olemassa olevaan niin helpoin tyhjentää näin
        harrastukset = new Harrastukset();

        hakemisto = nimi;
        jasenet.lueTiedostosta(nimi);
        harrastukset.lueTiedostosta(nimi);
    }


    /**
     * Tallettaa kerhon tiedot tiedostoon
     * @throws SailoException jos tallettamisessa ongelmia
     */
    public void tallenna() throws SailoException {
        String virhe = "";
        try {
            jasenet.tallenna(hakemisto);
        } catch ( SailoException ex ) {
            virhe = ex.getMessage();
        }

        try {
            harrastukset.tallenna(hakemisto);
        } catch ( SailoException ex ) {
            virhe += ex.getMessage();
        }
        if ( !"".equals(virhe) ) throw new SailoException(virhe);
    }


    
    /**
     * @param args ei käytössä
     */
    public static void main(String[] args)  {
        Kerho kerho = new Kerho();
        
        try {
            kerho.lueTiedostosta("koekelmit");
        } catch (SailoException ex) {
            System.out.println(ex.getMessage());
        }
         
        Jasen aku = new Jasen();
        Jasen aku2 = new Jasen();
        aku.rekisteroi();
        aku.vastaaAkuAnkka();     
        aku2.rekisteroi();
        aku2.vastaaAkuAnkka();    

        try {
            kerho.lisaa(aku);
            kerho.lisaa(aku2);

            for (int i=0; i<kerho.getJasenia(); i++) {
                Jasen jasen = kerho.annaJasen(i);
                jasen.tulosta(System.out);
            }
            
            kerho.tallenna();
        } catch (SailoException e) {
            // e.printStackTrace();
            System.err.println(e.getMessage());
        }

    }

}
